@@ -129,31 +129,33 @@ class Worker:
 
         log.info("finished forking all workers")
 
-        while len(self.children) > 0:
-            try:
-                self._pubsub = self.wakaq.broker.pubsub()
-                self._pubsub.subscribe(self.wakaq.broadcast_key)
+        try:
+            self._pubsub = self.wakaq.broker.pubsub()
+            self._pubsub.subscribe(self.wakaq.broadcast_key)
 
-                while not self._stop_processing:
-                    self._read_child_logs()
-                    self._check_max_mem_percent()
-                    self._refork_missing_children()
-                    self._enqueue_ready_eta_tasks()
+            while not self._stop_processing:
+                self._read_child_logs()
+                self._check_max_mem_percent()
+                self._refork_missing_children()
+                self._enqueue_ready_eta_tasks()
+                self._cleanup_children()
+                self._check_child_runtimes()
+                self._listen_for_broadcast_task()
+
+            if self._stop_processing:
+                if len(self.children) > 0:
+                    log.info("shutting down...")
+                while len(self.children) > 0:
                     self._cleanup_children()
                     self._check_child_runtimes()
-                    self._listen_for_broadcast_task()
+                    time.sleep(0.05)
 
-                if self._stop_processing:
-                    if len(self.children) > 0:
-                        log.info("shutting down...")
-                    while len(self.children) > 0:
-                        self._cleanup_children()
-                        self._check_child_runtimes()
-                        time.sleep(0.05)
-
-            except:
+        except:
+            try:
                 log.error(traceback.format_exc())
-                self._stop()
+            except:
+                print(traceback.format_exc())
+            self._stop()
 
     def _child(self, stdout, pingout, broadcastin):
         os.set_blocking(pingout, False)
@@ -199,7 +201,7 @@ class Worker:
                 queue_name, payload = self.wakaq._blocking_dequeue()
                 if payload is not None:
                     task = self.wakaq.tasks[payload["name"]]
-                    retry = payload["retry"]
+                    retry = payload.get("retry") or 0
                     try:
                         self._execute_task(task, payload)
                     except SoftTimeout:
@@ -308,9 +310,9 @@ class Worker:
 
     def _execute_broadcast_tasks(self):
         payloads = read_fd(self._broadcastin)
-        if payloads == b"":
+        if payloads == "":
             return
-        for payload in payloads.decode("utf8").splitlines():
+        for payload in payloads.splitlines():
             payload = deserialize(payload)
             task = self.wakaq.tasks[payload["name"]]
             retry = 0
@@ -329,8 +331,8 @@ class Worker:
     def _read_child_logs(self):
         for child in self.children:
             logs = read_fd(child.stdin)
-            if logs != b"":
-                log.handlers[0].stream.write(logs.decode("utf8"))
+            if logs != "":
+                log.handlers[0].stream.write(logs)
 
     def _check_max_mem_percent(self):
         if not self.wakaq.max_mem_percent:
@@ -357,7 +359,7 @@ class Worker:
     def _check_child_runtimes(self):
         for child in self.children:
             ping = read_fd(child.pingin)
-            if ping != b"":
+            if ping != "":
                 child.last_ping = time.time()
                 child.soft_timeout_reached = False
             elif self.wakaq.soft_timeout or self.wakaq.hard_timeout:
